@@ -2,7 +2,9 @@
 # coding=utf-8
 """A simple example demonstrating the following how to emit a non-zero exit code in your cmd2 application.
 """
-from typing import List
+from typing import (
+    List,
+)
 
 import cmd2
 
