@@ -7,10 +7,17 @@ import argparse
 import pytest
 
 import cmd2
-from cmd2 import Cmd2ArgumentParser, constants
-from cmd2.argparse_custom import generate_range_error
+from cmd2 import (
+    Cmd2ArgumentParser,
+    constants,
+)
+from cmd2.argparse_custom import (
+    generate_range_error,
+)
 
-from .conftest import run_cmd
+from .conftest import (
+    run_cmd,
+)
 
 
 class ApCustomTestApp(cmd2.Cmd):
@@ -260,3 +267,20 @@ def test_override_parser():
     # Verify DEFAULT_ARGUMENT_PARSER is now our CustomParser
     from examples.custom_parser import CustomParser
     assert DEFAULT_ARGUMENT_PARSER == CustomParser
+
+
+def test_apcustom_metavar_tuple():
+    # Test the case when a tuple metavar is used with nargs an integer > 1
+    parser = Cmd2ArgumentParser()
+    parser.add_argument('--aflag', nargs=2, metavar=('foo', 'bar'), help='This is a test')
+    assert '[--aflag foo bar]' in parser.format_help()
+
+
+def test_cmd2_attribute_wrapper():
+    initial_val = 5
+    wrapper = cmd2.Cmd2AttributeWrapper(initial_val)
+    assert wrapper.get() == initial_val
+
+    new_val = 22
+    wrapper.set(new_val)
+    assert wrapper.get() == new_val
