@@ -9,12 +9,22 @@ import io
 import os
 import sys
 import tempfile
-from code import InteractiveConsole
+from code import (
+    InteractiveConsole,
+)
 
 import pytest
 
 import cmd2
-from cmd2 import COMMAND_NAME, ansi, clipboard, constants, exceptions, plugin, utils
+from cmd2 import (
+    COMMAND_NAME,
+    ansi,
+    clipboard,
+    constants,
+    exceptions,
+    plugin,
+    utils,
+)
 
 from .conftest import (
     HELP_HISTORY,
@@ -47,6 +57,7 @@ def outsim_app():
 def test_version(base_app):
     assert cmd2.__version__
 
+@pytest.mark.skipif(sys.version_info >= (3, 8), reason="failing in CI systems for Python 3.8 and 3.9")
 def test_not_in_main_thread(base_app, capsys):
     import threading
     cli_thread = threading.Thread(name='cli_thread', target=base_app.cmdloop)
@@ -217,6 +228,23 @@ def test_shell_last_result(base_app):
     run_cmd(base_app, 'shell fake')
     assert base_app.last_result is not None
 
+
+def test_shell_manual_call(base_app):
+    # Verifies crash from Issue #986 doesn't happen
+    cmds = [
+        'echo "hi"',
+        'echo "there"',
+        'echo "cmd2!"'
+    ]
+    cmd = ';'.join(cmds)
+
+    base_app.do_shell(cmd)
+
+    cmd = '&&'.join(cmds)
+
+    base_app.do_shell(cmd)
+
+
 def test_base_py(base_app):
     # Make sure py can't edit Cmd.py_locals. It used to be that cmd2 was passing its py_locals
     # dictionary to the py environment instead of a shallow copy.
@@ -964,6 +992,16 @@ class HelpApp(cmd2.Cmd):
     def do_undoc(self, arg):
         pass
 
+    def do_multiline_docstr(self, arg):
+        """
+        This documentation
+        is multiple lines
+        and there are no
+        tabs
+        """
+        pass
+
+
 @pytest.fixture
 def help_app():
     app = HelpApp()
@@ -987,6 +1025,11 @@ def test_help_overridden_method(help_app):
     expected = normalize('This overrides the edit command and does nothing.')
     assert out == expected
 
+def test_help_multiline_docstring(help_app):
+    out, err = run_cmd(help_app, 'help multiline_docstr')
+    expected = normalize('This documentation\nis multiple lines\nand there are no\ntabs')
+    assert out == expected
+
 
 class HelpCategoriesApp(cmd2.Cmd):
     """Class for testing custom help_* methods which override docstring help."""
@@ -1224,7 +1267,7 @@ def test_select_ctrl_c(outsim_app, monkeypatch, capsys):
 class HelpNoDocstringApp(cmd2.Cmd):
     greet_parser = argparse.ArgumentParser()
     greet_parser.add_argument('-s', '--shout', action="store_true", help="N00B EMULATION MODE")
-    @cmd2.with_argparser_and_unknown_args(greet_parser)
+    @cmd2.with_argparser(greet_parser, with_unknown_args=True)
     def do_greet(self, opts, arg):
         arg = ''.join(arg)
         if opts.shout:
@@ -1243,24 +1286,6 @@ optional arguments:
   -s, --shout  N00B EMULATION MODE
 """
 
-@pytest.mark.skipif(sys.platform.startswith('win'),
-                    reason="utils.which function only used on Mac and Linux")
-def test_which_editor_good():
-    editor = cmd2.Cmd.DEFAULT_EDITOR
-    path = utils.which(editor)
-
-    # Assert that the editor was found because some editor should exist on all Mac and Linux systems
-    assert path
-
-@pytest.mark.skipif(sys.platform.startswith('win'),
-                    reason="utils.which function only used on Mac and Linux")
-def test_which_editor_bad():
-    nonexistent_editor = 'this_editor_does_not_exist.exe'
-    path = utils.which(nonexistent_editor)
-    # Assert that the non-existent editor wasn't found
-    assert path is None
-
-
 class MultilineApp(cmd2.Cmd):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, multiline_commands=['orate'], **kwargs)
@@ -1268,7 +1293,7 @@ class MultilineApp(cmd2.Cmd):
     orate_parser = argparse.ArgumentParser()
     orate_parser.add_argument('-s', '--shout', action="store_true", help="N00B EMULATION MODE")
 
-    @cmd2.with_argparser_and_unknown_args(orate_parser)
+    @cmd2.with_argparser(orate_parser, with_unknown_args=True)
     def do_orate(self, opts, arg):
         arg = ''.join(arg)
         if opts.shout:
@@ -1607,16 +1632,32 @@ def test_alias_create(base_app):
     out, err = run_cmd(base_app, 'alias list fake')
     assert out == normalize('alias create fake run_pyscript')
 
-def test_alias_create_with_quoted_value(base_app):
-    """Demonstrate that quotes in alias value will be preserved (except for redirectors and terminators)"""
+    # Overwrite alias
+    out, err = run_cmd(base_app, 'alias create fake help')
+    assert out == normalize("Alias 'fake' overwritten")
+
+    # Look up the updated alias
+    out, err = run_cmd(base_app, 'alias list fake')
+    assert out == normalize('alias create fake help')
+
+    # Test silent flag
+    out, err = run_cmd(base_app, 'alias create --silent fake set')
+    assert not out
+
+    out, err = run_cmd(base_app, 'alias list --with_silent fake')
+    assert out == normalize('alias create --silent fake set')
+
+def test_alias_create_with_quoted_tokens(base_app):
+    """Demonstrate that quotes in alias value will be preserved"""
+    create_command = 'alias create fake help ">" "out file.txt" ";"'
 
     # Create the alias
-    out, err = run_cmd(base_app, 'alias create fake help ">" "out file.txt" ";"')
+    out, err = run_cmd(base_app, create_command)
     assert out == normalize("Alias 'fake' created")
 
-    # Look up the new alias (Only the redirector should be unquoted)
+    # Look up the new alias and verify all quotes are preserved
     out, err = run_cmd(base_app, 'alias list fake')
-    assert out == normalize('alias create fake help > "out file.txt" ;')
+    assert out == normalize(create_command)
 
 @pytest.mark.parametrize('alias_name', invalid_command_name)
 def test_alias_create_invalid_name(base_app, alias_name, capsys):
@@ -1701,15 +1742,32 @@ def test_macro_create(base_app):
     out, err = run_cmd(base_app, 'macro list fake')
     assert out == normalize('macro create fake run_pyscript')
 
-def test_macro_create_with_quoted_value(base_app):
-    """Demonstrate that quotes in macro value will be preserved (except for redirectors and terminators)"""
+    # Overwrite macro
+    out, err = run_cmd(base_app, 'macro create fake help')
+    assert out == normalize("Macro 'fake' overwritten")
+
+    # Look up the updated macro
+    out, err = run_cmd(base_app, 'macro list fake')
+    assert out == normalize('macro create fake help')
+
+    # Test silent flag
+    out, err = run_cmd(base_app, 'macro create --silent fake set')
+    assert not out
+
+    out, err = run_cmd(base_app, 'macro list --with_silent fake')
+    assert out == normalize('macro create --silent fake set')
+
+def test_macro_create_with_quoted_tokens(base_app):
+    """Demonstrate that quotes in macro value will be preserved"""
+    create_command = 'macro create fake help ">" "out file.txt" ";"'
+
     # Create the macro
-    out, err = run_cmd(base_app, 'macro create fake help ">" "out file.txt" ";"')
+    out, err = run_cmd(base_app, create_command)
     assert out == normalize("Macro 'fake' created")
 
-    # Look up the new macro (Only the redirector should be unquoted)
+    # Look up the new macro and verify all quotes are preserved
     out, err = run_cmd(base_app, 'macro list fake')
-    assert out == normalize('macro create fake help > "out file.txt" ;')
+    assert out == normalize(create_command)
 
 @pytest.mark.parametrize('macro_name', invalid_command_name)
 def test_macro_create_invalid_name(base_app, macro_name):
@@ -2230,7 +2288,7 @@ def test_disable_and_enable_category(disable_commands_app):
 
     # Make sure neither function completes
     text = ''
-    line = 'has_helper_funcs'
+    line = 'has_helper_funcs {}'.format(text)
     endidx = len(line)
     begidx = endidx - len(text)
 
@@ -2238,7 +2296,7 @@ def test_disable_and_enable_category(disable_commands_app):
     assert first_match is None
 
     text = ''
-    line = 'has_no_helper_funcs'
+    line = 'has_no_helper_funcs {}'.format(text)
     endidx = len(line)
     begidx = endidx - len(text)
 
@@ -2274,7 +2332,7 @@ def test_disable_and_enable_category(disable_commands_app):
 
     # has_helper_funcs should complete now
     text = ''
-    line = 'has_helper_funcs'
+    line = 'has_helper_funcs {}'.format(text)
     endidx = len(line)
     begidx = endidx - len(text)
 
@@ -2283,7 +2341,7 @@ def test_disable_and_enable_category(disable_commands_app):
 
     # has_no_helper_funcs had no completer originally, so there should be no results
     text = ''
-    line = 'has_no_helper_funcs'
+    line = 'has_no_helper_funcs {}'.format(text)
     endidx = len(line)
     begidx = endidx - len(text)
 
@@ -2341,15 +2399,24 @@ def test_disabled_message_command_name(disable_commands_app):
     out, err = run_cmd(disable_commands_app, 'has_helper_funcs')
     assert err[0].startswith('has_helper_funcs is currently disabled')
 
-
-def test_startup_script(request):
+@pytest.mark.parametrize('silent_startup_script', [
+    True,
+    False
+])
+def test_startup_script(request, capsys, silent_startup_script):
     test_dir = os.path.dirname(request.module.__file__)
     startup_script = os.path.join(test_dir, '.cmd2rc')
-    app = cmd2.Cmd(allow_cli_args=False, startup_script=startup_script)
+    app = cmd2.Cmd(allow_cli_args=False, startup_script=startup_script, silent_startup_script=silent_startup_script)
     assert len(app._startup_commands) == 1
-    assert app._startup_commands[0] == "run_script {}".format(utils.quote_string(startup_script))
     app._startup_commands.append('quit')
     app.cmdloop()
+
+    out, err = capsys.readouterr()
+    if silent_startup_script:
+        assert not out
+    else:
+        assert out
+
     out, err = run_cmd(app, 'alias list')
     assert len(out) > 1
     assert 'alias create ls' in out[0]
