@@ -20,10 +20,18 @@ import sys
 import pytest
 
 import cmd2
-from cmd2 import utils
-from examples.subcommands import SubcommandsExample
+from cmd2 import (
+    utils,
+)
+from examples.subcommands import (
+    SubcommandsExample,
+)
 
-from .conftest import complete_tester, normalize, run_cmd
+from .conftest import (
+    complete_tester,
+    normalize,
+    run_cmd,
+)
 
 # List of strings used with completion functions
 food_item_strs = ['Pizza', 'Ham', 'Ham Sandwich', 'Potato', 'Cheese "Pizza"']
@@ -483,8 +491,6 @@ def test_path_completion_doesnt_match_wildcards(cmd2_app, request):
     # Currently path completion doesn't accept wildcards, so will always return empty results
     assert cmd2_app.path_complete(text, line, begidx, endidx) == []
 
-@pytest.mark.skipif(sys.platform == 'win32', reason="getpass.getuser() does not work on Windows in AppVeyor because "
-                                                    "no user name environment variables are set")
 def test_path_completion_complete_user(cmd2_app):
     import getpass
     user = getpass.getuser()
@@ -972,6 +978,7 @@ def test_complete_set_value(cmd2_app):
 
     first_match = complete_tester(text, line, begidx, endidx, cmd2_app)
     assert first_match == "SUCCESS "
+    assert cmd2_app.completion_hint == "Hint:\n  value  a settable param\n"
 
 def test_complete_set_value_invalid_settable(cmd2_app, capsys):
     text = ''
