@@ -9,7 +9,12 @@ import sys
 import pytest
 
 import cmd2
-from cmd2 import Cmd2ArgumentParser, exceptions, plugin, with_argparser
+from cmd2 import (
+    Cmd2ArgumentParser,
+    exceptions,
+    plugin,
+    with_argparser,
+)
 
 # Python 3.5 had some regressions in the unitest.mock module, so use 3rd party mock if available
 try:
@@ -279,7 +284,7 @@ class PluggedApp(Plugin, cmd2.Cmd):
     @with_argparser(parser)
     def do_argparse_cmd(self, namespace: argparse.Namespace):
         """Repeat back the arguments"""
-        self.poutput(namespace.__statement__)
+        self.poutput(namespace.cmd2_statement.get())
 
 ###
 #
