@@ -3,6 +3,7 @@
 """
 Unit testing for cmd2/utils.py module.
 """
+import os
 import signal
 import sys
 import time
@@ -10,7 +11,15 @@ import time
 import pytest
 
 import cmd2.utils as cu
-from cmd2.constants import HORIZONTAL_ELLIPSIS
+from cmd2.constants import (
+    HORIZONTAL_ELLIPSIS,
+)
+
+try:
+    import mock
+except ImportError:
+    from unittest import mock
+
 
 HELLO_WORLD = 'Hello, world!'
 
@@ -154,6 +163,13 @@ def test_stdsim_read(stdout_sim):
     assert stdout_sim.read() == my_str
     assert stdout_sim.getvalue() == ''
 
+    stdout_sim.write(my_str)
+
+    assert stdout_sim.getvalue() == my_str
+    assert stdout_sim.read(2) == my_str[:2]
+    assert stdout_sim.getvalue() == my_str[2:]
+
+
 def test_stdsim_read_bytes(stdout_sim):
     b_str = b'Hello World'
     stdout_sim.buffer.write(b_str)
@@ -627,3 +643,19 @@ def test_str_to_bool_invalid():
 def test_str_to_bool_bad_input():
     with pytest.raises(ValueError):
         cu.str_to_bool(1)
+
+def test_find_editor_specified():
+    expected_editor = os.path.join('fake_dir', 'editor')
+    with mock.patch.dict(os.environ, {'EDITOR': expected_editor}):
+        editor = cu.find_editor()
+    assert editor == expected_editor
+
+def test_find_editor_not_specified():
+    # Use existing path env setting. Something in the editor list should be found.
+    editor = cu.find_editor()
+    assert editor
+
+    # Overwrite path env setting with invalid path, clear all other env vars so no editor should be found.
+    with mock.patch.dict(os.environ, {'PATH': 'fake_dir'}, clear=True):
+        editor = cu.find_editor()
+    assert editor is None
